import logging
from typing import Any

class Logger:
    def disable(self, name: str, loglevel=logging.CRITICAL) -> None: ...
    def setName(self, name: str) -> None: ...
    def info(self, message, *args, serialize=True, **kwargs) -> None: ...
    def debug(self, message, *args, serialize=True, **kwargs) -> None: ...
    def warning(self, message, *args, **kwargs) -> None: ...
    def setLevel(self, level) -> None: ...
    def addHandler(self, handler) -> None: ...
    def error(self, message, *args, serialize=False, **kwargs) -> None: ...
    def critical(
        self, message, *args, serialize=False, stacktrace=False, **kwargs
    ) -> None: ...
    def logger(self) -> Any: ...
