"""NavConfig Meta information."""

__title__ = "navconfig"
__description__ = (
    "Configuration tool for all Navigator Services "
    "Tool for accessing Config info from different sources."
)
__version__ = "1.9.2"
__copyright__ = "Copyright (c) 2020-2024 Jesus Lara"
__author__ = "Jesus Lara"
__author_email__ = "jesuslarag@gmail.com"
__license__ = "MIT"
